<?php

// Картинка.
$image = imagecreatefromjpeg('image.jpg');

// Маска.
$mask = imagecreatefrompng('mask.png');

// Тень.
$shadow = imagecreatefrompng('shadow.png');

// Размер тени сверху.
$shadowTopWidth = 5;

// Размер тени слева.
$shadowLeftWidth = 5;

// Цвет границы в формате RGBA.
$borderColor = array(250, 150, 0, 0);

// Толщина границы.
$borderWidth = 3;

/**************************************************************************************************************/

function canvas($width, $height, array $color = NULL)
{
	$canvas = imagecreatetruecolor(
		$width ? $width : $this->width, 
		$height ? $height : $this->height
	);
	
	imagealphablending($canvas, false);
	imagesavealpha($canvas, true);

	if(!$color)
	$color = array(0,0,0,127);

	$color = imagecolorallocatealpha(
		$canvas, $color[0], $color[1], $color[2], count($color) > 3 ? $color[3] : 0
	);

	imagecolortransparent($canvas, $color);
	imagefill($canvas, 0, 0, $color);    

	return $canvas;
}

function resize(&$source, $width, $height, $normalize = false)
{
	$x = 0;
	$curWidth = imagesx($source);
	$curHeight = imagesy($source);
	$newImage = canvas($width, $height);
	
	if(
		$normalize && (
			$curWidth > $width ||
			$curHeight > $height
		)
	)
	{
		$widthCopy = $width;
		
		if($curHeight > $height)
		$width = $height / $curHeight * $curWidth;
		
		$x = $width != $widthCopy ? ($widthCopy - $width) / 2 : 0;
	}

	imagecopyresampled($newImage, $source, $x, 0, 0, 0, $width, $height, $curWidth, $curHeight);
	$source = $newImage;
}

/**************************************************************************************************************/

imagesavealpha($mask, true);
imagesavealpha($shadow, true);

resize($image, imagesx($mask), imagesy($mask), true);
resize($shadow, imagesx($shadow) + $borderWidth * 2, imagesy($shadow) + $borderWidth * 2);

$width = imagesx($image);
$height = imagesy($image);

$border = canvas($width, $height);


for($x = 0; $x < $width; $x++)
{
	for($y = 0; $y < $height; $y++)
	{
		$maskRGBA = array_values(imagecolorsforindex($mask, imagecolorat($mask, $x, $y)));
		$imageRGBA = array_values(imagecolorsforindex($image, imagecolorat($image, $x, $y)));
		
		$_color = imagecolorallocatealpha($mask, $imageRGBA[0], $imageRGBA[1], $imageRGBA[2], $maskRGBA[3]);
		
		$_borderColor = imagecolorallocatealpha(
			$border, $borderColor[0], $borderColor[1], $borderColor[2],
			$maskRGBA[3] < $borderColor[3] ? $borderColor[3] : $maskRGBA[3]
		);

		imagesetpixel($image, $x, $y, $_color);
		imagesetpixel($border, $x, $y, $_borderColor);
	}
}

resize($border, $width + $borderWidth * 2, $height + $borderWidth * 2);
imagealphablending($border, true);
imagecopy($border, $image, $borderWidth, $borderWidth, 0, 0, $width, $height);

// Накладываем картинку на тень.
imagealphablending($shadow, true);
imagecopy($shadow, $border, $shadowTopWidth, $shadowLeftWidth, 0, 0, $width + $borderWidth * 2, $height + $borderWidth * 2);

$image = $shadow;

header('Content-Type: image/png');
imagepng($image);


